"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var helpers_1 = require("./helpers");
/**
 * Applies a pipeline to an array of values.
 * It works only with async pipelines
 *
 * ```
 * const pipe1 = pipe(useCallValue()).pipe(multiplyBy(2)).pipe(add(11));
 * const pipe2 = pipe(useCallValue()).pipe(applyEach(pipe1));
 *
 * pipe2.resolve([10, 20, 30]) // Promise([31, 51, 71])
 * ```
 * @export
 * @template T
 * @template R
 * @param {IPipe<T, R>} pipe
 * @returns {(arg: T[], context: IContext<T[], R>) => Promise<R>[]}
 */
function applyEach(pipe) {
    return function (values, context) {
        helpers_1.throwContextExecutionFlow('applyEach', context, 'async');
        helpers_1.throwIfNotArray('applyEach', values);
        return values.map(function (value) { return pipe.resolve(value); });
    };
}
exports.applyEach = applyEach;
/**
 * Applies a pipeline to an array of values.
 * It works only with sync pipelines
 *
 * ```
 * const pipe1 = pipe(useCallValue()).pipe(multiplyBy(2)).pipe(add(11));
 * const pipe2 = pipe(useCallValue()).pipe(applyEach(pipe1));
 *
 * pipe2.resolveSync([10, 20, 30]) // [31, 51, 71]
 * ```
 * @export
 * @template T
 * @template R
 * @param {IPipe<T, R>} pipe
 * @returns {(arg: T[], context: IContext<T[], R>) => R[]}
 */
function applyEachSync(pipe) {
    return function (values, context) {
        helpers_1.throwContextExecutionFlow('applyEachSync', context, 'sync');
        helpers_1.throwIfNotArray('applyEach', values);
        return values.map(function (value) { return pipe.resolveSync(value); });
    };
}
exports.applyEachSync = applyEachSync;
/**
 * Concatenates previous value with another array.
 *
 * If the previous value is not an array an error will be thrown
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(concat([4,5,6]))
 *   .resolveSync([1,2,3]) // [1,2,3,4,5,6]
 * ```
 *
 * @export
 * @template T
 * @param {T[]} newArr
 * @returns {ExplicitCallable<T[], T[]>}
 */
function concat(newArr) {
    return function (arr) {
        helpers_1.throwIfNotArray('concat', arr);
        return arr.concat(newArr);
    };
}
exports.concat = concat;
/**
 * Filters the previous value with a given callback. The callback must return a boolean value.
 *
 * If the previous value is not an array an error will be thrown
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(filter((a, index, array) => array.length > 2 && index === 1 && a > 1))
 *   .resolveSync([1,2,3]) // [2]
 * ```
 * @export
 * @template T
 * @param {(arg: T, index: number, arr: T[]) => boolean} filterCallback
 * @returns {ExplicitCallable<T[], unknown[]>}
 */
function filter(filterCallback) {
    return function (arr) {
        helpers_1.throwIfNotArray('filter', arr);
        return arr.filter(filterCallback);
    };
}
exports.filter = filter;
/**
 * Filters the previous value with a given value.
 *
 * If the previous value is not an array an error will be thrown
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(filterWith(2))
 *   .resolveSync([1,2,3,2,3,2]) // [2,2,2]
 * ```
 * @export
 * @template T
 * @param {T} value
 * @returns {ExplicitCallable<T[], T[]>}
 */
function filterWith(value) {
    return function (arr) {
        helpers_1.throwIfNotArray('filterWith', arr);
        return arr.filter(function (x) { return x === value; });
    };
}
exports.filterWith = filterWith;
/**
 * Finds a value as specified by a find callback. The callback must return `true`.
 *
 * If the value is not found, the pipe will return an `undefined`
 *
 * If the previous value is not an array an error will be thrown
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(find((arg: number, index, array) => index > 2 && index < array.length - 1 && arg > 2))
 *   .resolveSync([1,2,3,2,3,2]) // 3
 * ```
 * @export
 * @template T
 * @param {(arg: T, index: number, arr: T[]) => boolean} callback
 * @returns {(ExplicitCallable<T[], T | undefined>)}
 */
function find(callback) {
    return function (arr) {
        helpers_1.throwIfNotArray('findExact', arr);
        for (var i = 0; i < arr.length; i++) {
            if (callback(arr[i], i, arr)) {
                return arr[i];
            }
        }
    };
}
exports.find = find;
/**
 * Finds an exacts value.
 *
 * If the value is not found, the pipe will return an `undefined`
 *
 * If the previous value is not an array an error will be thrown
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(find(1))
 *   .resolveSync([1,2,3,2,3,2]) // 1
 * ```
 * @export
 * @template T
 * @param {T} value
 * @returns {(ExplicitCallable<T[], T | undefined>)}
 */
function findExact(value) {
    return function (arr) {
        helpers_1.throwIfNotArray('findExact', arr);
        for (var i = 0; i < arr.length; i++) {
            if (arr[i] === value) {
                return arr[i];
            }
        }
    };
}
exports.findExact = findExact;
/**
 * Joins the previous value with a given char.
 *
 * If the previous value is not an array an error will be thrown
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(join('*'))
 *   .resolveSync([1,2,3]) // '1*2*3'
 * ```
 * @export
 * @param {string} char
 * @returns {ExplicitCallable<unknown[], string>}
 */
function join(char) {
    return function (arr) {
        helpers_1.throwIfNotArray('applyEach', arr);
        return arr.join(char);
    };
}
exports.join = join;
/**
 * Returns the length of the previous value.
 *
 * If the previous value is not an array an error will be thrown
 *
 * ```
 * pipe(useCallValue())
 *  .pipe(length())
 *  .resolveSync([1,2,3]) // 3
 * ```
 * @export
 * @returns {ExplicitCallable<unknown[], number>}
 */
function length() {
    return function (arr) {
        helpers_1.throwIfNotArray('length', arr);
        return arr.length;
    };
}
exports.length = length;
/**
 * Returns the nth element in the previous value.
 *
 * If the previous value is not an array an error will be thrown
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(nth(3))
 *   .resolveSync([0, 2, 5, 12, 24]) // 12
 * ```
 *
 * @export
 * @param {number} index
 * @returns {Callable}
 */
function nth(index) {
    return function (arg) {
        helpers_1.throwIfNotArray('nth', arg);
        return arg[index];
    };
}
exports.nth = nth;
/**
 * Reverses the previous value.
 *
 * If the previous value is not an array an error will be thrown
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(reverse())
 *   .resolveSync([1,2,3]) // [3,2,1]
 * ```
 *
 * @export
 * @returns {ExplicitCallable<unknown[], unknown[]>}
 */
function reverse() {
    return function (arr) {
        helpers_1.throwIfNotArray('reverse', arr);
        var newArr = arr.slice();
        var length = newArr.length;
        for (var i = length / 2; i > 0; i--) {
            var startIndex = length - i;
            var left = newArr[startIndex];
            var right = newArr[i];
            newArr[startIndex] = right;
            newArr[i] = left;
        }
        return arr;
    };
}
exports.reverse = reverse;
