"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var helpers_1 = require("./helpers");
/**
 * Is the same of the `switch` construct.
 *
 * ```
 * // silly example: checking if a string is inside an array of strings
 * const cities = pipe(useCallValue())
 *   .pipe(
 *     createSwitch(
 *       createSwitchDefault('City not found'),
 *       createSwitchOption('Munich', 'Beerfest!'),
 *       createSwitchOption('Rome', 'We love carbonara'),
 *       createSwitchOption('London', 'God save the queen'),
 *     )
 *   )
 *   .toSyncFunction();
 *
 * cities('Munich') // 'Beerfest!'
 * cities('Rome') // 'We love carbonara'
 * cities('London') // 'God save the queen'
 * cities('Casalpusterlengo') // 'City not found'
 * ```
 *
 * To create the default, you can use the `createSwitchDefault` method, whilst using `createSwitchOption` you can add a switch case.
 * @export
 * @template TValue
 * @param {...SwitchOption<TValue>[]} args
 * @returns {(arg: TValue) => unknown}
 */
function createSwitch() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return function (arg) {
        var defaultCase;
        var tests = args.slice();
        while (tests.length > 0) {
            var result = tests.pop()(arg);
            if (result.default) {
                defaultCase = result;
                continue;
            }
            if (result.success) {
                return result.value;
            }
        }
        if (defaultCase !== undefined) {
            return defaultCase.value;
        }
    };
}
exports.createSwitch = createSwitch;
/**
 * Creates a `default` case for the `createSwitch` function
 *
 * @export
 * @template Value
 * @param {Value} value
 * @returns {SwitchOption<Value>}
 */
function createSwitchDefault(value) {
    return function (arg) {
        return {
            default: true,
            success: false,
            value: value,
        };
    };
}
exports.createSwitchDefault = createSwitchDefault;
/**
 * Creates a `switch case` for the `createSwitch` function
 *
 * @export
 * @template Match
 * @template Value
 * @param {Match} match
 * @param {Value} value
 * @returns {SwitchOption<Value>}
 */
function createSwitchOption(match, value) {
    return function (arg) {
        if (arg === match) {
            return { default: false, success: true, value: value };
        }
        return { default: false, success: false, value: undefined };
    };
}
exports.createSwitchOption = createSwitchOption;
/**
 * This function is the same of the `if/else` statement.
 *
 * It takes two arguments, the `left` and the `right` part. The left part is intended as the `false` comparison result, while the `right` is the `true`.
 *
 * ```
 * const smsLengthCheck = pipe(useCallValue())
 *   .pipe(isGreaterThan(144))
 *   .pipe(fold('', 'Maximum character'))
 *   .toSyncFunction();
 *
 * smsLengthCheck('lorem') // ''
 * smsLengthCheck('lorem'.repeat(2000)) // 'Maximum character'
 * ```
 * @export
 * @template Left
 * @template Right
 * @param {Left} left
 * @param {Right} right
 * @returns {<T extends boolean>(arg: T) => T extends true ? Right : Left}
 */
function fold(left, right) {
    return function (arg) { return (arg ? right : left); };
}
exports.fold = fold;
/**
 * This function works like the `if/else` statement.
 *
 * It requires three arguments:
 *
 * * a Condition (a `function` which returns a `boolean`)
 * * a Left, which can be a value or another `pipe`. If is a pipe, it will be resolved using the context's async/sync flow
 * * a Right, which can be a value or another `pipe`. If is a pipe, it will be resolved using the context's async/sync flow
 *
 * ```
 * const simple = pipe(useCallValue())
 *   .pipe(
 *     logical.ifElse(
 *       (arg: number) => arg > 5,
 *       'lower than 5',
 *       'greater than 5'
 *     )
 *   )
 *   .toSyncFunction();
 *
 * const complex = pipe(useCallValue())
 *   .pipe(
 *     logical.ifElse(
 *       (arg: number) => arg > 5,
 *       pipe(useCallValue()).pipe(math.pow(2)),
 *       pipe(useCallValue()).pipe(math.divideBy(2)),
 *     )
 *   ).toSyncFunction();
 *
 * simple(4) // lower than 5
 * simple(10) // greater than 5
 * complex(4) // 14
 * complex(10) // 5
 * ```
 *
 * @export
 * @template TCondition
 * @template Left
 * @template Right
 * @template K
 * @param {TCondition} condition
 * @param {Left} left
 * @param {Right} right
 * @returns {<T extends K>(arg: T, context: IContext) => ReturnType<TCondition> extends true ? Right : Left}
 */
function ifElse(condition, left, right) {
    var resolveIfIsPipe = function (context, currentPipe, value) {
        return helpers_1.isContextFlowAsync(context) ? currentPipe.resolve(value) : currentPipe.resolveSync(value);
    };
    return function (value, context) {
        helpers_1.throwIfNotFunction("ifElse", 'condition', condition);
        var result = context.call(condition);
        if (helpers_1.isPipe(left) && !result) {
            return resolveIfIsPipe(context, left, value);
        }
        else if (helpers_1.isPipe(right) && result) {
            return resolveIfIsPipe(context, right, value);
        }
        else if (result) {
            return right;
        }
        else {
            return left;
        }
    };
}
exports.ifElse = ifElse;
