"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var helpers_1 = require("./helpers");
/**
 * Sums a value to the previous one.
 *
 * ```
 * pipe(useCallValue()).pipe(add(1)).resolveSync(10) // 11
 * ```
 *
 * @export
 * @param {number} value
 * @returns {ExplicitCallable<number, number>}
 */
function add(value) {
    return function (arg) {
        helpers_1.throwIfNotNumber('add', arg);
        return arg + value;
    };
}
exports.add = add;
/**
 * Changes previous value sign, from positive to negative and vice-versa.
 *
 * ```
 * pipe(put(-123)).pipe(changeSign()) // 123
 * ```
 *
 * @export
 * @returns {ExplicitCallable<number, number>}
 */
function changeSign() {
    return function (arg) {
        helpers_1.throwIfNotNumber('changeSign', arg);
        return -arg;
    };
}
exports.changeSign = changeSign;
/**
 * Decrements the previous value by one.
 *
 * ```
 * pipe(useCallValue()).pipe(decrement()).resolve(44) // 41
 * ```
 *
 * @export
 * @returns {ExplicitCallable<number, number>}
 */
function decrement() {
    return function (arg) {
        helpers_1.throwIfNotNumber('decrement', arg);
        return --arg;
    };
}
exports.decrement = decrement;
/**
 * Divides the previous value by the passed one.
 *
 * ```
 * pipe(useCallValue()).pipe(divideBy(2)).resolve(44) // 22
 * ```
 *
 * @export
 * @param {number} value
 * @returns {ExplicitCallable<number, number>}
 */
function divideBy(value) {
    return function (arg) {
        helpers_1.throwIfNotNumber('divideBy', arg);
        return arg / value;
    };
}
exports.divideBy = divideBy;
/**
 * Increments the previous value by one.
 *
 * ```
 * pipe(useCallValue()).pipe(increment()).resolve(44) // 45
 * ```
 *
 * @export
 * @returns {ExplicitCallable<number, number>}
 */
function increment() {
    return function (arg) {
        helpers_1.throwIfNotNumber('increment', arg);
        return ++arg;
    };
}
exports.increment = increment;
/**
 * Multiplies the previous value by the given one
 *
 * ```
 * pipe(useCallValue()).pipe(multiplyBy(2)).resolve(44) // 88
 * ```
 *
 * @export
 * @param {number} value
 * @returns {ExplicitCallable<number, number>}
 */
function multiplyBy(value) {
    return function (arg) {
        helpers_1.throwIfNotNumber('multiplyBy', arg);
        return arg * value;
    };
}
exports.multiplyBy = multiplyBy;
/**
 * Converts previous' value from positive sign to negative unless is already negative.
 *
 * ```
 * pipe(useCallValue()).pipe(negative()).resolve(44) // -44
 * pipe(useCallValue()).pipe(negative()).resolve(-12) // -12
 * ```
 *
 * @export
 * @returns {ExplicitCallable<number, number>}
 */
function negative() {
    return function (arg) {
        helpers_1.throwIfNotNumber('negative', arg);
        return arg > 0 ? -arg : arg;
    };
}
exports.negative = negative;
/**
 * Converts previous' value from negative sign to positive unless is already positive.
 *
 * ```
 * pipe(useCallValue()).pipe(positive()).resolve(44) // 44
 * pipe(useCallValue()).pipe(positive()).resolve(-12) // 12
 * ```
 *
 * @export
 * @returns {ExplicitCallable<number, number>}
 */
function positive() {
    return function (arg) {
        helpers_1.throwIfNotNumber('positive', arg);
        return arg > 0 ? arg : -arg;
    };
}
exports.positive = positive;
/**
 * Raises to power the previous value by a given exponent
 *
 * ```
 * pipe(useCallValue()).pipe(pow(4)).resolve(2) // 16
 * pipe(useCallValue()).pipe(pow(2)).resolve(-2) // 4
 * ```
 *
 * @export
 * @param {number} exponent
 * @returns {ExplicitCallable<number, number>}
 */
function pow(exponent) {
    return function (arg) {
        helpers_1.throwIfNotNumber('pow', arg);
        return Math.pow(arg, exponent);
    };
}
exports.pow = pow;
/**
 * Extracts the root of the previous value by a given exponent
 *
 * ```
 * pipe(useCallValue()).pipe(root(2)).resolve(4) // 2
 * pipe(useCallValue()).pipe(root(2)).resolve(9) // 3
 * ```
 *
 * @export
 * @param {number} exponent
 * @returns {ExplicitCallable<number, number>}
 */
function root(exponent) {
    return function (arg) {
        helpers_1.throwIfNotNumber('root', arg);
        return Math.pow(arg, 1 / exponent);
    };
}
exports.root = root;
/**
 * Subtracts the previous value by the given one
 *
 * ```
 * pipe(useCallValue()).pipe(subtract(2)).resolve(9) // 7
 * ```
 *
 * @export
 * @param {number} value
 * @returns {ExplicitCallable<number, number>}
 */
function subtract(value) {
    return function (arg) {
        helpers_1.throwIfNotNumber('subtract', arg);
        return arg - value;
    };
}
exports.subtract = subtract;
/**
 * Takes all numbers in a number array between two values (inclusive)
 *
 * ```
 * pipe(useCallValue()).pipe(takeBetween(5, 7)).resolve([4, 5, 6, 7, 8]) // [5, 6, 7]
 * ```
 *
 * @export
 * @param {number} start
 * @param {number} end
 * @returns {ExplicitCallable<number[], number[]>}
 */
function takeBetween(start, end) {
    return function (arg) {
        helpers_1.throwIfNotArray("takeBetween", arg);
        var newArray = [];
        var length = arg.length;
        for (var i = 0; i < length; i++) {
            var value = arg[i];
            helpers_1.throwIfNotNumber('takeBetween', value);
            if (value >= start && value <= end) {
                newArray.push(value);
            }
        }
        return newArray;
    };
}
exports.takeBetween = takeBetween;
/**
 * Returns the greater number in a number array
 *
 * ```
 * pipe(useCallValue()).pipe(takeGreater()).resolve([4, 5, 44, 6, 7, 8]) // 44
 * ```
 *
 * @export
 * @returns {ExplicitCallable<number[], number>}
 */
function takeGreater() {
    return function (arg) {
        helpers_1.throwIfNotArray('takeGreater', arg);
        return Math.max.apply(Math, arg);
    };
}
exports.takeGreater = takeGreater;
/**
 * Takes all numbers in a number array greater than a given value.
 *
 * ```
 * pipe(useCallValue()).pipe(takeGreaterThan(8)).resolve([4, 5, 44, 6, 7, 8]) // [44]
 * ```
 *
 * This function accepts a second parameter (boolean) to include also the same value
 *
 * ```
 * pipe(useCallValue()).pipe(takeGreaterThan(8, true)).resolve([4, 5, 44, 6, 7, 8]) // [8, 44]
 * ```
 *
 * @export
 * @param {number} check
 * @param {boolean} [equal=false]
 * @returns {ExplicitCallable<number[], number[]>}
 */
function takeGreaterThan(check, equal) {
    if (equal === void 0) { equal = false; }
    return function (arg) {
        helpers_1.throwIfNotArray("takeGreaterThan", arg);
        var newArray = [];
        var length = arg.length;
        for (var i = 0; i < length; i++) {
            var value = arg[i];
            helpers_1.throwIfNotNumber('takeGreaterThan', value);
            if (equal && value >= check) {
                newArray.push(value);
            }
            else if (value > check) {
                newArray.push(value);
            }
        }
        return newArray;
    };
}
exports.takeGreaterThan = takeGreaterThan;
/**
 * Returns the lower number in a number array
 *
 * ```
 * pipe(takeLower()).pipe(takeGreater()).resolve([4, 5, 44, 6, 7, 8]) // 4
 * ```
 *
 * @export
 * @returns {ExplicitCallable<number[], number>}
 */
function takeLower() {
    return function (arg) {
        helpers_1.throwIfNotArray('takeLower', arg);
        return Math.min.apply(Math, arg);
    };
}
exports.takeLower = takeLower;
/**
 * Takes all numbers in a number array lower than a given value.
 *
 * ```
 * pipe(useCallValue()).pipe(takeLowerThan(5)).resolve([4, 5, 44, 6, 7, 8]) // [4]
 * ```
 *
 * This function accepts a second parameter (boolean) to include also the same value
 *
 * ```
 * pipe(useCallValue()).pipe(takeLowerThan(8, true)).resolve([4, 5, 44, 6, 7, 8]) // [4, 5, 6, 7, 8]
 * ```
 *
 * @export
 * @param {number} check
 * @param {boolean} [equal=false]
 * @returns {ExplicitCallable<number[], number[]>}
 */
function takeLowerThan(check, equal) {
    if (equal === void 0) { equal = false; }
    return function (arg) {
        helpers_1.throwIfNotArray("takeLowerThan", arg);
        var newArray = [];
        var length = arg.length;
        for (var i = 0; i < length; i++) {
            var value = arg[i];
            helpers_1.throwIfNotNumber('takeLowerThan', value);
            if (equal && value <= check) {
                newArray.push(value);
            }
            else if (value < check) {
                newArray.push(value);
            }
        }
        return newArray;
    };
}
exports.takeLowerThan = takeLowerThan;
/**
 * Takes all numbers in a number array lower than the first passed value or greater than the second passed value. Matching elements are discarded.
 *
 * ```
 * pipe(useCallValue()).pipe(takeOuter(5, 10)).resolveSync([3,4,5,10,11]) // [3, 4, 11]
 * ```
 *
 * @export
 * @param {number} start
 * @param {number} end
 * @returns {ExplicitCallable<number[], number[]>}
 */
function takeOuter(start, end) {
    return function (arg) {
        helpers_1.throwIfNotArray("takeOuter", arg);
        var newArray = [];
        var length = arg.length;
        for (var i = 0; i < length; i++) {
            var value = arg[i];
            helpers_1.throwIfNotNumber('takeOuter', value);
            if (value < start || value > end) {
                newArray.push(value);
            }
        }
        return newArray;
    };
}
exports.takeOuter = takeOuter;
