import { ExplicitCallable, Callable } from './pipe';
/**
 * Returns previous value except for the given keys. This applies only to objects.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(exclude('foo'))
 *   .resolveSync({ foo: 123, bar: 'baz' }); // { bar: 'baz' }
 * ```
 *
 * @export
 * @template KObject
 * @template Keys
 * @param {...Keys[]} keys
 * @returns {ExplicitCallable<KObject, Pick<KObject, Keys>>}
 */
export declare function exclude<KObject, Keys extends keyof KObject = keyof KObject>(...keys: Keys[]): ExplicitCallable<KObject, Pick<KObject, Keys>>;
/**
 * Returns if an object has a owned property
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(hasProperty('foo'))
 *   .resolveSync({ foo: new Date() }) // true
 * ```
 *
 * @export
 * @param {string} propertyKey
 * @returns {ExplicitCallable<unknown, boolean>}
 */
export declare function hasProperty(propertyKey: string): ExplicitCallable<unknown, boolean>;
/**
 * Returns previous's value keys. Works only for objects
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(keys())
 *   .resolveSync({ bar: 123, foo: new Date() }) // ['bar', 'foo']
 * ```
 *
 * @export
 * @returns {Callable}
 */
export declare function keys(): Callable;
/**
 * Returns a property key value by a given path. This applies only to objects.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(maybe('foo.bar'))
 *   .resolveSync({ foo: { bar: 100 } }) // 100
 * ```
 *
 * You can provide a fallback value or a fallback pipeline if the path does not match the object schema.
 *
 * If you start you pipeline with the `useCallValue` function, the monad will be invoked with an `undefined` value.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(maybe('foo.bar.baz', 123))
 *   .resolveSync({ foo: { bar: 100 } }) // 123
 *
 * // or
 *
 * const fallback = pipe(useCallValue());
 *
 * pipe(useCallValue())
 *   .pipe(maybe('foo.bar.baz', fallback))
 *   .resolveSync({ foo: { bar: 100 } }) // undefined
 *
 * // or
 *
 * const fallback = pipe(put(10));
 *
 * pipe(useCallValue())
 *   .pipe(maybe('foo.bar.baz', fallback))
 *   .resolveSync({ foo: { bar: 100 } }) // 10
 * ```
 *
 * @export
 * @template TObject
 * @template TReturn
 * @param {string} path
 * @param {TReturn} [fallbackValue]
 * @returns {(ExplicitCallable<TObject, TReturn | undefined>)}
 */
export declare function maybe<TObject = unknown, TReturn = unknown>(path: string, fallbackValue?: TReturn): ExplicitCallable<TObject, TReturn | undefined>;
/**
 * Merges the previous object with the given one
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(merge({ foo: 'bar' }))
 *   .resolveSync({ bar: 'baz' }) // { foo: 'bar', bar: 'baz' }
 * ```
 *
 * @export
 * @template T
 * @template K
 * @param {T} target
 * @returns {(ExplicitCallable<K, K & T>)}
 */
export declare function merge<T extends object, K>(target: T): ExplicitCallable<K, K & T>;
/**
 * Returns a new object (previous value) with the given keys. This applies only to objects.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(pick('foo'))
 *   .resolveSync({ foo: 123, bar: 'baz' }); // { foo: 123 }
 * ```
 *
 * @export
 * @template KObject
 * @template Keys
 * @param {...Keys[]} keys
 * @returns {ExplicitCallable<KObject, Pick<KObject, Keys>>}
 */
export declare function pick<KObject, Keys extends keyof KObject = keyof KObject>(...keys: Keys[]): ExplicitCallable<KObject, Pick<KObject, Keys>>;
