"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var helpers_1 = require("./helpers");
var util_1 = require("util");
/**
 * Returns previous value except for the given keys. This applies only to objects.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(exclude('foo'))
 *   .resolveSync({ foo: 123, bar: 'baz' }); // { bar: 'baz' }
 * ```
 *
 * @export
 * @template KObject
 * @template Keys
 * @param {...Keys[]} keys
 * @returns {ExplicitCallable<KObject, Pick<KObject, Keys>>}
 */
function exclude() {
    var keys = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        keys[_i] = arguments[_i];
    }
    return function (arg) {
        helpers_1.throwIfNotObject("pick", arg);
        var newObject = __assign({}, arg);
        var keysArray = keys.slice();
        while (keysArray.length > 0) {
            delete newObject[keysArray.pop()];
        }
        return newObject;
    };
}
exports.exclude = exclude;
/**
 * Returns if an object has a owned property
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(hasProperty('foo'))
 *   .resolveSync({ foo: new Date() }) // true
 * ```
 *
 * @export
 * @param {string} propertyKey
 * @returns {ExplicitCallable<unknown, boolean>}
 */
function hasProperty(propertyKey) {
    return function (arg) {
        if (util_1.isNullOrUndefined(arg)) {
            return false;
        }
        return Object.prototype.hasOwnProperty.call(arg, propertyKey);
    };
}
exports.hasProperty = hasProperty;
/**
 * Returns previous's value keys. Works only for objects
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(keys())
 *   .resolveSync({ bar: 123, foo: new Date() }) // ['bar', 'foo']
 * ```
 *
 * @export
 * @returns {Callable}
 */
function keys() {
    return function (arg) {
        helpers_1.throwIfNotObject("keys", arg);
        return Object.keys(arg);
    };
}
exports.keys = keys;
/**
 * Returns a property key value by a given path. This applies only to objects.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(maybe('foo.bar'))
 *   .resolveSync({ foo: { bar: 100 } }) // 100
 * ```
 *
 * You can provide a fallback value or a fallback pipeline if the path does not match the object schema.
 *
 * If you start you pipeline with the `useCallValue` function, the monad will be invoked with an `undefined` value.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(maybe('foo.bar.baz', 123))
 *   .resolveSync({ foo: { bar: 100 } }) // 123
 *
 * // or
 *
 * const fallback = pipe(useCallValue());
 *
 * pipe(useCallValue())
 *   .pipe(maybe('foo.bar.baz', fallback))
 *   .resolveSync({ foo: { bar: 100 } }) // undefined
 *
 * // or
 *
 * const fallback = pipe(put(10));
 *
 * pipe(useCallValue())
 *   .pipe(maybe('foo.bar.baz', fallback))
 *   .resolveSync({ foo: { bar: 100 } }) // 10
 * ```
 *
 * @export
 * @template TObject
 * @template TReturn
 * @param {string} path
 * @param {TReturn} [fallbackValue]
 * @returns {(ExplicitCallable<TObject, TReturn | undefined>)}
 */
function maybe(path, fallbackValue) {
    var shouldUseFallback = function (current) {
        return util_1.isNullOrUndefined(current) && !util_1.isNullOrUndefined(fallbackValue);
    };
    var returnFallbackValue = function (context) {
        if (!helpers_1.isPipe(fallbackValue)) {
            return fallbackValue;
        }
        var isAsync = context.executionFlow === 'async';
        if (isAsync) {
            return fallbackValue.resolve(undefined);
        }
        return fallbackValue.resolveSync(undefined);
    };
    return function (arg, context) {
        helpers_1.throwIfNotObject('maybe', arg);
        var pathChunks = path.split('.').reverse();
        var current = arg;
        while (pathChunks.length) {
            var key = pathChunks.pop();
            if (!current) {
                break;
            }
            current = current[key];
        }
        if (shouldUseFallback(current)) {
            return returnFallbackValue(context);
        }
        return current;
    };
}
exports.maybe = maybe;
/**
 * Merges the previous object with the given one
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(merge({ foo: 'bar' }))
 *   .resolveSync({ bar: 'baz' }) // { foo: 'bar', bar: 'baz' }
 * ```
 *
 * @export
 * @template T
 * @template K
 * @param {T} target
 * @returns {(ExplicitCallable<K, K & T>)}
 */
function merge(target) {
    return function (arg) {
        helpers_1.throwIfNotObject('merge', arg);
        return __assign({}, arg, target);
    };
}
exports.merge = merge;
/**
 * Returns a new object (previous value) with the given keys. This applies only to objects.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(pick('foo'))
 *   .resolveSync({ foo: 123, bar: 'baz' }); // { foo: 123 }
 * ```
 *
 * @export
 * @template KObject
 * @template Keys
 * @param {...Keys[]} keys
 * @returns {ExplicitCallable<KObject, Pick<KObject, Keys>>}
 */
function pick() {
    var keys = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        keys[_i] = arguments[_i];
    }
    return function (arg) {
        helpers_1.throwIfNotObject("pick", arg);
        var newObject = {};
        var keysArray = keys.slice();
        while (keysArray.length > 0) {
            var key = keysArray.pop();
            if (arg[key]) {
                newObject[key] = arg[key];
            }
        }
        return newObject;
    };
}
exports.pick = pick;
