export declare type Callable = <Input, Output>(input: Input, context: IContext<unknown, Input>) => Output;
export declare type ExecutionContextFlow = 'async' | 'sync';
export declare type ExplicitCallable<Input, Output> = (input: Input, context: IContext<unknown, Input>) => Output;
export declare type ResolvedPipe<Input, Output> = (input: Input) => Promise<Output>;
export declare type ResolvedSyncPipe<Input, Output> = (input: Input) => Output;
export declare type Pipeline = (Callable | ExplicitCallable<unknown, unknown>)[];
export interface IContext<CallValue = unknown, PreviousValue = unknown> {
    /**
     * The value used to call the pipeline
     * @type {CallValue}
     * @memberof IContext
     */
    readonly callValue: CallValue;
    /**
     * The context execution flow can be `sync` or `async`, based on
     * the invokation of sync methods like `resolveSync` and `toSyncFunction`
     * @type {ExecutionContextFlow}
     * @memberof IContext
     */
    readonly executionFlow: ExecutionContextFlow;
    /**
     * Current mutation index.
     * @type {number}
     * @memberof IContext
     */
    readonly mutationIndex: number;
    /**
     * Previous mutated value.
     * @type {PreviousValue}
     * @memberof IContext
     */
    readonly previousValue: PreviousValue;
    /**
     * An array including previous values
     * @type {unknown[]}
     * @memberof IContext
     */
    readonly previousValues: unknown[];
    /**
     * Calls a function with the `previousValue` and `context` as arguments
     * @template ReturnType
     * @param {ExplicitCallable<PreviousValue, ReturnType>} callable
     * @returns {ReturnType}
     * @memberof IContext
     */
    call<ReturnType>(callable: ExplicitCallable<PreviousValue, ReturnType>): ReturnType;
    /**
     * Applies a mutation.
     * @param {Pipeline} pipeline
     * @returns {IMutatedContext}
     * @memberof IContext
     */
    mutate?(pipeline: Pipeline): IMutatedContext;
}
export interface IPipe<InitialCallable, Output> {
    /**
     * Determines if the function is memoized or not
     * @type {boolean}
     * @memberof IPipe
     */
    readonly memoized: boolean;
    /**
     * @type {'pipe'}
     * @memberof IPipe
     */
    readonly type: 'pipe';
    /**
     * Method for chaining a function
     * @template NextValue
     * @param {ExplicitCallable<Output, NextValue>} callable
     * @returns {IPipe<InitialCallable, NextValue>}
     * @memberof IPipe
     */
    pipe<NextValue>(callable: ExplicitCallable<Output, NextValue>): IPipe<InitialCallable, NextValue>;
    /**
     * Resolves the pipeline asynchronously.
     * @type {ResolvedPipe<InitialCallable, Output>}
     * @memberof IPipe
     */
    resolve: ResolvedPipe<InitialCallable, Output>;
    /**
     * Resolves the pipeline synchronously.
     * @type {ResolvedSyncPipe<InitialCallable, Output>}
     * @memberof IPipe
     */
    resolveSync: ResolvedSyncPipe<InitialCallable, Output>;
    /**
     * Creates an `async` function from the pipeline
     * @returns {(input: InitialCallable) => Promise<Output>}
     * @memberof IPipe
     */
    toFunction(): (input: InitialCallable) => Promise<Output>;
    /**
     * Creates a function from the pipeline
     * @returns {(input: InitialCallable) => Output}
     * @memberof IPipe
     */
    toSyncFunction(): (input: InitialCallable) => Output;
}
export interface IMutatedContext {
    /**
     * A list of new mutations to append after the current one has been applied.
     * @type {Pipeline}
     * @memberof IMutatedContext
     */
    pipeline: Pipeline;
}
/**
 * Creates a new pipeline
 * @export
 * @template CallValue
 * @template NextValue
 * @param {ExplicitCallable<CallValue, NextValue>} callable
 * @param {boolean} [memoized=false]
 * @returns
 */
export declare function pipe<CallValue, NextValue>(callable: ExplicitCallable<CallValue, NextValue>, memoized?: boolean): IPipe<CallValue, NextValue>;
