import { ExplicitCallable } from './pipe';
/**
 * Returns previous value in camel-case. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *    .pipe(camelCase())
 *    .resolveSync('hello world') // 'helloWorld'
 * ```
 *
 * @export
 * @returns {ExplicitCallable<string, string>}
 */
export declare function camelCase(): ExplicitCallable<string, string>;
/**
 * Returns previous value as an array of chars. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(chars())
 *   .resolveSync('hello') // returns ['h', 'e', 'l', 'l', 'o']
 * ```
 *
 * @export
 * @returns {ExplicitCallable<string, string[]>}
 */
export declare function chars(): ExplicitCallable<string, string[]>;
/**
 * Concatenate previous value with another string. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(concat('world'))
 *   .resolveSync('hello') // 'helloworld'
 * ```
 *
 * @export
 * @param {string} str
 * @returns {ExplicitCallable<string, string>}
 */
export declare function concat(str: string): ExplicitCallable<string, string>;
/**
 * Returns if the previous value contains a portion of text. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(includes('llo'))
 *   .resolveSync('hello') // true
 * ```
 *
 * @export
 * @param {string} str
 * @returns {ExplicitCallable<string, boolean>}
 */
export declare function includes(str: string): ExplicitCallable<string, boolean>;
/**
 * Returns previous value length. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(length())
 *   .resolveSync('hello') // 5
 * ```
 *
 * @export
 * @returns {ExplicitCallable<string, number>}
 */
export declare function length(): ExplicitCallable<string, number>;
/**
 * Returns previous value in lower case. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(lowercase())
 *   .resolveSync('HELLO') // 'hello'
 * ```
 *
 * @export
 * @returns {ExplicitCallable<string, string>}
 */
export declare function lowercase(): ExplicitCallable<string, string>;
/**
 * Returns previous value in pascal-case. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(pascalCase())
 *   .resolveSync('hello world') // 'HelloWorld'
 * ```
 *
 * @export
 * @returns {ExplicitCallable<string, string>}
 */
export declare function pascalCase(): ExplicitCallable<string, string>;
/**
 * Repeats previous value a number of times. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(repeat())
 *   .resolveSync('hello') // hellohello
 *
 * pipe(useCallValue())
 *   .pipe(repeat(2))
 *   .resolveSync('hello') // hellohellohello
 * ```
 *
 * @export
 * @param {number} [count=1]
 * @returns {ExplicitCallable<string, string>}
 */
export declare function repeat(count?: number): ExplicitCallable<string, string>;
/**
 * Replaces all occurencies from the previous value. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(replaceAll('l', '1'))
 *   .resolveSync('hello') // he110
 * ```
 *
 * @export
 * @param {string} needle
 * @param {string} replaceWith
 * @returns {ExplicitCallable<string, string>}
 */
export declare function replaceAll(needle: string, replaceWith: string): ExplicitCallable<string, string>;
/**
 * Returns previous value in a binary representation. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(toBinary())
 *   .resolveSync('hello world') // [ '1101000', '1100101', '1101100', '1101100', '1101111', '100000', '1110111', '1101111', '1110010', '1101100', '1100100' ]
 * ```
 *
 * @export
 * @returns {ExplicitCallable<string, string[]>}
 */
export declare function toBinaryArray(): ExplicitCallable<string, string[]>;
/**
 * Returns previous value in upper case. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(uppercase())
 *   .resolveSync('hello') // 'HELLO'
 * ```
 *
 * @export
 * @returns {ExplicitCallable<string, string>}
 */
export declare function uppercase(): ExplicitCallable<string, string>;
