"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var helpers_1 = require("./helpers");
var regexps = {};
/**
 * Returns previous value in camel-case. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *    .pipe(camelCase())
 *    .resolveSync('hello world') // 'helloWorld'
 * ```
 *
 * @export
 * @returns {ExplicitCallable<string, string>}
 */
function camelCase() {
    return function (arg) {
        helpers_1.throwIfNotString("camelCase", arg);
        return arg
            .replace(/(?:^\w|[A-Z]|\b\w)/g, function (letter, index) { return (index === 0 ? letter.toLowerCase() : letter.toUpperCase()); })
            .replace(/\s+/g, '');
    };
}
exports.camelCase = camelCase;
/**
 * Returns previous value as an array of chars. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(chars())
 *   .resolveSync('hello') // returns ['h', 'e', 'l', 'l', 'o']
 * ```
 *
 * @export
 * @returns {ExplicitCallable<string, string[]>}
 */
function chars() {
    return function (arg) {
        helpers_1.throwIfNotString("chars", arg);
        return arg.split('');
    };
}
exports.chars = chars;
/**
 * Concatenate previous value with another string. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(concat('world'))
 *   .resolveSync('hello') // 'helloworld'
 * ```
 *
 * @export
 * @param {string} str
 * @returns {ExplicitCallable<string, string>}
 */
function concat(str) {
    return function (arg) {
        helpers_1.throwIfNotString("concat", arg);
        return arg + str;
    };
}
exports.concat = concat;
/**
 * Returns if the previous value contains a portion of text. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(includes('llo'))
 *   .resolveSync('hello') // true
 * ```
 *
 * @export
 * @param {string} str
 * @returns {ExplicitCallable<string, boolean>}
 */
function includes(str) {
    return function (arg) {
        helpers_1.throwIfNotString('find', arg);
        return arg.indexOf(str) >= 0;
    };
}
exports.includes = includes;
/**
 * Returns previous value length. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(length())
 *   .resolveSync('hello') // 5
 * ```
 *
 * @export
 * @returns {ExplicitCallable<string, number>}
 */
function length() {
    return function (arg) {
        helpers_1.throwIfNotString("length", arg);
        return arg.length;
    };
}
exports.length = length;
/**
 * Returns previous value in lower case. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(lowercase())
 *   .resolveSync('HELLO') // 'hello'
 * ```
 *
 * @export
 * @returns {ExplicitCallable<string, string>}
 */
function lowercase() {
    return function (arg) {
        helpers_1.throwIfNotString("lowercase", arg);
        return arg.toLowerCase();
    };
}
exports.lowercase = lowercase;
/**
 * Returns previous value in pascal-case. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(pascalCase())
 *   .resolveSync('hello world') // 'HelloWorld'
 * ```
 *
 * @export
 * @returns {ExplicitCallable<string, string>}
 */
function pascalCase() {
    return function (arg, context) {
        var camelized = camelCase()(arg, context);
        return camelized.charAt(0).toUpperCase() + camelized.slice(1);
    };
}
exports.pascalCase = pascalCase;
/**
 * Repeats previous value a number of times. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(repeat())
 *   .resolveSync('hello') // hellohello
 *
 * pipe(useCallValue())
 *   .pipe(repeat(2))
 *   .resolveSync('hello') // hellohellohello
 * ```
 *
 * @export
 * @param {number} [count=1]
 * @returns {ExplicitCallable<string, string>}
 */
function repeat(count) {
    if (count === void 0) { count = 1; }
    return function (arg) {
        helpers_1.throwIfNotString('repeat', arg);
        var i = 0;
        var newArg = arg;
        while (i < count) {
            newArg += arg;
            i++;
        }
        return newArg;
    };
}
exports.repeat = repeat;
/**
 * Replaces all occurencies from the previous value. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(replaceAll('l', '1'))
 *   .resolveSync('hello') // he110
 * ```
 *
 * @export
 * @param {string} needle
 * @param {string} replaceWith
 * @returns {ExplicitCallable<string, string>}
 */
function replaceAll(needle, replaceWith) {
    return function (arg) {
        helpers_1.throwIfNotString("replaceAll", arg);
        regexps[needle] = regexps[needle] || new RegExp(needle, 'gi');
        return arg.replace(regexps[needle], replaceWith);
    };
}
exports.replaceAll = replaceAll;
/**
 * Returns previous value in a binary representation. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(toBinary())
 *   .resolveSync('hello world') // [ '1101000', '1100101', '1101100', '1101100', '1101111', '100000', '1110111', '1101111', '1110010', '1101100', '1100100' ]
 * ```
 *
 * @export
 * @returns {ExplicitCallable<string, string[]>}
 */
function toBinaryArray() {
    var binarizeCharset = function (index, charset) {
        if (index >= charset.length) {
            return charset;
        }
        var binary = charset[index].charCodeAt(0).toString(2);
        var newCharset = charset.slice();
        newCharset.splice(index, 1, binary);
        return binarizeCharset(index + 1, newCharset);
    };
    return function (arg, context) {
        helpers_1.throwIfNotString("toBinary", arg);
        return binarizeCharset(0, context.call(chars()));
    };
}
exports.toBinaryArray = toBinaryArray;
/**
 * Returns previous value in upper case. Previous value must be a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(uppercase())
 *   .resolveSync('hello') // 'HELLO'
 * ```
 *
 * @export
 * @returns {ExplicitCallable<string, string>}
 */
function uppercase() {
    return function (arg) {
        helpers_1.throwIfNotString("uppercase", arg);
        return arg.toUpperCase();
    };
}
exports.uppercase = uppercase;
