"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var helpers_1 = require("./helpers");
/**
 * Throws if the previous value is not of the same type expected.
 *
 * TypeName is the second portion of a `Object.prototype.toString` call in **lowerCase**:
 *
 * > [object **TypeName**] --> `typename`
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(exactTypeOf('date'))
 *   .resolveSync(123) // throws!
 *
 * pipe(useCallValue())
 *   .pipe(exactTypeOf('date'))
 *   .resolveSync(new Date()) // 2019-03-26T02:17:000Z
 * ```
 *
 * @export
 * @param {string} typeName
 * @returns {Callable}
 */
function exactTypeOf(typeName) {
    return function (arg) {
        if (helpers_1.getTypeOf(arg) === typeName) {
            return arg;
        }
        throw new TypeError("Invalid type " + helpers_1.getTypeOf(arg) + ", expected " + typeName);
    };
}
exports.exactTypeOf = exactTypeOf;
/**
 * Throws if the previous value is not of the same type expected.
 *
 * Internally uses the `typeof` operator.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(exactTypeOf('number'))
 *   .resolveSync(123) // 123
 *
 * pipe(useCallValue())
 *   .pipe(exactTypeOf('number'))
 *   .resolveSync(`hello world!`) // throws!
 * ```
 *
 * @export
 * @template T
 * @param {T} typeName
 * @returns {Callable}
 */
function ofType(typeName) {
    return function (arg) {
        if (typeof arg === typeName) {
            return arg;
        }
        throw new TypeError("Invalid type " + typeof arg + ", expected " + typeName);
    };
}
exports.ofType = ofType;
/**
 * Converts previous value to an array.
 *
 * ```
 * pipe(useCallValue()).pipe(toArray()).resolveSync(10) // [10]
 * ```
 *
 * @export
 * @returns {<T>(arg: T) => T[]}
 */
function toArray() {
    return function (arg) { return [arg]; };
}
exports.toArray = toArray;
/**
 * Converts previous value to a boolean value.
 *
 * ```
 * pipe(useCallValue()).pipe(toBoolean()).resolveSync(10) // true
 * pipe(useCallValue()).pipe(toBoolean()).resolveSync(0) // false
 * pipe(useCallValue()).pipe(toBoolean()).resolveSync(null) // false
 * pipe(useCallValue()).pipe(toBoolean()).resolveSync(undefined) // false
 * pipe(useCallValue()).pipe(toBoolean()).resolveSync('') // false
 * pipe(useCallValue()).pipe(toBoolean()).resolveSync('123') // true
 * ```
 *
 * @export
 * @returns {<T>(arg: T) => boolean}
 */
function toBoolean() {
    return function (arg) { return (!arg === true ? false : true); };
}
exports.toBoolean = toBoolean;
/**
 * Converts previous value to a Date instance.
 *
 * ```
 * pipe(useCallValue())
 *    .pipe(toDate())
 *    .resolveSync(`2019-01-01T00:00:000Z`) // Date(2019, 0, 1, 0, 0, 0)
 * ```
 *
 * @export
 * @returns {((arg: string | number) => Date)}
 */
function toDate() {
    return function (arg) { return new Date(arg); };
}
exports.toDate = toDate;
/**
 * Converts previous value to a number.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(toNumber())
 *   .resolveSync('12000') // 12000
 * ```
 *
 * @export
 * @returns {(arg: unknown) => number}
 */
function toNumber() {
    return function (arg) { return Number(arg); };
}
exports.toNumber = toNumber;
/**
 * Converts previous value to a set. Previous value must be an array.
 *
 * @export
 * @returns {<T>(arg: T[]) => Set<T>}
 */
function toSet() {
    return function (arg) {
        helpers_1.throwIfNotArray('toSet', arg);
        return new Set(arg);
    };
}
exports.toSet = toSet;
/**
 * Converts previous value to a string.
 *
 * ```
 * pipe(useCallValue())
 *   .pipe(toString())
 *   .resolveSync([1,2,3]) // "1,2,3"
 * ```
 *
 * @export
 * @returns {(arg: unknown) => string}
 */
function toString() {
    return function (arg) { return String(arg); };
}
exports.toString = toString;
